@@ -1,17 +1,36 @@
-/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* ECellDate - Date item for e-table.
- * Copyright (C) 2001 Ximian, Inc.
- * Author: Chris Lahey <clahey@ximian.com>
+/* 
+ * e-cell-date.c - Date item for e-table.
+ * Copyright 2001, Ximian, Inc.
+ *
+ * Author:
+ *  Chris Lahey <clahey@ximian.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
+
+#include "e-cell-date.h"
+
 #include <sys/time.h>
 #include <time.h>
 #include <unistd.h>
+#include <string.h>
 #include <gal/util/e-util.h>
 #include <gal/widgets/e-unicode.h>
 #include <gal/util/e-i18n.h>
-#include "e-cell-date.h"
 
 #define PARENT_TYPE e_cell_text_get_type ()
 
@@ -25,6 +44,7 @@ ecd_get_text(ECellText *cell, ETableModel *model, int col, int row)
 	time_t yesdate;
 	struct tm then, now, yesterday;
 	char buf[26];
+	char *temp, *ret_val;
 	gboolean done = FALSE;
 
 	if (date == 0) {
@@ -33,19 +53,35 @@ ecd_get_text(ECellText *cell, ETableModel *model, int col, int row)
 
 	localtime_r (&date, &then);
 	localtime_r (&nowdate, &now);
-	if (then.tm_mday == now.tm_mday &&
-	    then.tm_mon == now.tm_mon &&
-	    then.tm_year == now.tm_year) {
-		strftime (buf, 26, _("Today %l:%M %p"), &then);
+
+	if (nowdate - date < 60 * 60 * 8 && nowdate > date) {
+		e_strftime_fix_am_pm (buf, 26, _("%l:%M %p"), &then);
 		done = TRUE;
 	}
+
+	if (!done) {
+		if (then.tm_mday == now.tm_mday &&
+		    then.tm_mon == now.tm_mon &&
+		    then.tm_year == now.tm_year) {
+			e_strftime_fix_am_pm (buf, 26, _("Today %l:%M %p"), &then);
+			done = TRUE;
+		}
+	}
 	if (!done) {
 		yesdate = nowdate - 60 * 60 * 24;
 		localtime_r (&yesdate, &yesterday);
 		if (then.tm_mday == yesterday.tm_mday &&
 		    then.tm_mon == yesterday.tm_mon &&
 		    then.tm_year == yesterday.tm_year) {
-			strftime (buf, 26, _("Yesterday %l:%M %p"), &then);
+#if 0
+			if (nowdate - date < 60 * 60 * 12) {
+				e_strftime_fix_am_pm (buf, 26, _("Late Yesterday %l:%M %p"), &then);
+			} else {
+#endif
+				e_strftime_fix_am_pm (buf, 26, _("Yesterday %l:%M %p"), &then);
+#if 0
+			}
+#endif
 			done = TRUE;
 		}
 	}
@@ -57,7 +93,7 @@ ecd_get_text(ECellText *cell, ETableModel *model, int col, int row)
 			if (then.tm_mday == yesterday.tm_mday &&
 			    then.tm_mon == yesterday.tm_mon &&
 			    then.tm_year == yesterday.tm_year) {
-				strftime (buf, 26, _("%a %l:%M %p"), &then);
+				e_strftime_fix_am_pm (buf, 26, _("%a %l:%M %p"), &then);
 				done = TRUE;
 				break;
 			}
@@ -65,9 +101,9 @@ ecd_get_text(ECellText *cell, ETableModel *model, int col, int row)
 	}
 	if (!done) {
 		if (then.tm_year == now.tm_year) {
-			strftime (buf, 26, _("%b %d %l:%M %p"), &then);
+			e_strftime_fix_am_pm (buf, 26, _("%b %d %l:%M %p"), &then);
 		} else {
-			strftime (buf, 26, _("%b %d %Y"), &then);
+			e_strftime_fix_am_pm (buf, 26, _("%b %d %Y"), &then);
 		}
 	}
 #if 0
@@ -77,8 +113,14 @@ ecd_get_text(ECellText *cell, ETableModel *model, int col, int row)
 	ctime_r (&date, buf);
 #endif
 #endif
-	
-	return e_utf8_from_locale_string (buf);
+	temp = buf;
+	while ((temp = strstr (temp, "  "))) {
+		memmove (temp, temp + 1, strlen (temp));
+	}
+	temp = e_strdup_strip (buf);
+	ret_val = e_utf8_from_locale_string (temp);
+	g_free (temp);
+	return ret_val;
 }
 
 static void
