@@ -1,11 +1,11 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* e-cell-text.c - Text cell renderer
- * Copyright (C) 2000 Ximian, Inc.
+/*
+ * e-cell-text.c: Text cell renderer.
+ * Copyright 1999, 2000, 2001, Ximian, Inc.
  *
- * Authors: Miguel de Icaza <miguel@ximian.com>
- *          Chris Lahey <clahey@ximian.com>
- *
- * (C) 1999, 2000 Ximian, Inc.
+ * Authors:
+ *   Miguel de Icaza <miguel@ximian.com>
+ *   Chris Lahey <clahey@ximian.com>
  *
  * A lot of code taken from:
  *
@@ -19,9 +19,19 @@
  *
  * Author: Federico Mena <federico@nuclecu.unam.mx>
  *
- * TODO:
- *   Clean up UTF-8 handling
- *   UTF-8 selection
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
@@ -50,6 +60,13 @@
 #include "gal/util/e-text-event-processor-emacs-like.h"
 #include "e-table-tooltip.h"
 
+#define d(x)
+
+#if d(!)0
+#define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)), g_print ("%s: e_table_item_leave_edit\n", __FUNCTION__))
+#else
+#define e_table_item_leave_edit_(x) (e_table_item_leave_edit((x)))
+#endif
 
 #define ECT_CLASS(c) (E_CELL_TEXT_CLASS(GTK_OBJECT((c))->klass))
 
@@ -158,8 +175,6 @@ struct _CellEdit {
 
 	ETextEventProcessor *tep;       /* Text Event Processor */
 
-	/* Hmmm... this should probably be in native encoding? */
-
 	GtkWidget *invisible;           /* For selection handling */
 	gboolean has_selection;         /* TRUE if we have the selection */
 	gchar *primary_selection;       /* Primary selection text */
@@ -211,8 +226,8 @@ static GdkColor* e_cell_text_get_color (ECellTextView *cell_view, gchar *color_s
 
 static ECellClass *parent_class;
 
-static char *
-ect_get_text (ECellText *cell, ETableModel *model, int col, int row)
+char *
+e_cell_text_get_text (ECellText *cell, ETableModel *model, int col, int row)
 {
 	if (ECT_CLASS(cell)->get_text)
 		return ECT_CLASS(cell)->get_text (cell, model, col, row);
@@ -220,13 +235,21 @@ ect_get_text (ECellText *cell, ETableModel *model, int col, int row)
 		return NULL;
 }
 
-static void
-ect_free_text (ECellText *cell, char *text)
+void
+e_cell_text_free_text (ECellText *cell, char *text)
 {
 	if (ECT_CLASS(cell)->free_text)
 		ECT_CLASS(cell)->free_text (cell, text);
 }
 
+void
+e_cell_text_set_value (ECellText *cell, ETableModel *model, int col, int row,
+		       const char *text)
+{
+	if (ECT_CLASS(cell)->set_value)
+		ECT_CLASS(cell)->set_value (cell, model, col, row, text);
+}
+
 static char *
 ect_real_get_text (ECellText *cell, ETableModel *model, int col, int row)
 {
@@ -238,6 +261,17 @@ ect_real_free_text (ECellText *cell, char *text)
 {
 }
 
+/* This is the default method for setting the ETableModel value based on
+   the text in the ECellText. This simply uses the text as it is - it assumes
+   the value in the model is a char*. Subclasses may parse the text into
+   data structures to pass to the model. */
+static void
+ect_real_set_value (ECellText *cell, ETableModel *model, int col, int row,
+		    const char *text)
+{
+	e_table_model_set_value_at (model, col, row, text);
+}
+
 static void
 ect_queue_redraw (ECellTextView *text_view, int view_col, int view_row)
 {
@@ -246,39 +280,26 @@ ect_queue_redraw (ECellTextView *text_view, int view_col, int view_row)
 		view_col, view_row, view_col, view_row);
 }
 
-/*
- * Accept the currently edited text.  if it's the same as what's in the cell, do nothing.
- */
-static void
-ect_accept_edits (ECellTextView *text_view)
-{
-	CurrentCell *cell = (CurrentCell *) text_view->edit;
-
-	if (strcmp (text_view->edit->old_text, cell->text)) {
-		e_table_model_set_value_at (text_view->cell_view.e_table_model,
-					    cell->model_col, cell->row, cell->text);
-	}
-}
-
 /*
  * Shuts down the editing process
  */
 static void
-ect_stop_editing (ECellTextView *text_view)
+ect_stop_editing (ECellTextView *text_view, gboolean commit)
 {
 	CellEdit *edit = text_view->edit;
-	int row, view_col;
+	int row, view_col, model_col;
+	char *old_text, *text;
+	CurrentCell *cell = (CurrentCell *) text_view->edit;
 
 	if (!edit)
 		return;
 
-	row = edit->cell.row;
-	view_col = edit->cell.view_col;
+	row = cell->row;
+	view_col = cell->view_col;
+	model_col = cell->model_col;
 	
-	g_free (edit->old_text);
-	edit->old_text = NULL;
-	g_free (edit->cell.text);
-	edit->cell.text = NULL;
+	old_text = edit->old_text;
+	text = cell->text;
 	if (edit->invisible)
 		gtk_widget_unref (edit->invisible);
 	if (edit->tep)
@@ -302,8 +323,23 @@ ect_stop_editing (ECellTextView *text_view)
 	}
 
 	g_free (edit);
-	
+
 	text_view->edit = NULL;
+	if (commit) {
+		/*
+		 * Accept the currently edited text.  if it's the same as what's in the cell, do nothing.
+		 */
+		ECellView *ecell_view = (ECellView *) text_view;
+		ECellText *ect = (ECellText *) ecell_view->ecell;
+
+		if (strcmp (old_text, text)) {
+			e_cell_text_set_value (ect, ecell_view->e_table_model,
+					       model_col, row, text);
+		}
+	}
+	g_free (text);
+	g_free (old_text);
+
 	ect_queue_redraw (text_view, view_col, row);
 }
 
@@ -313,8 +349,8 @@ ect_stop_editing (ECellTextView *text_view)
 static void
 ect_cancel_edit (ECellTextView *text_view)
 {
-	ect_stop_editing (text_view);
-	e_table_item_leave_edit (text_view->cell_view.e_table_item_view);
+	ect_stop_editing (text_view, FALSE);
+	e_table_item_leave_edit_ (text_view->cell_view.e_table_item_view);
 }
 
 /*
@@ -750,8 +786,9 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 	GtkWidget *canvas = GTK_WIDGET (text_view->canvas);
 	gint return_val = 0;
 	CurrentCell cell, *cellptr;
+	d(gboolean press = FALSE);
 
-	if (flags & !E_CELL_EDITING)
+	if (!(flags & E_CELL_EDITING))
 		return 0;
 
 	build_current_cell (&cell, text_view, model_col, view_col, row);
@@ -794,7 +831,7 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 		if (edit_display) {
 			GdkEventKey key = event->key;
 			if (key.keyval == GDK_KP_Enter || key.keyval == GDK_Return){
-				e_table_item_leave_edit (text_view->cell_view.e_table_item_view);
+				e_table_item_leave_edit_ (text_view->cell_view.e_table_item_view);
 			} else {
 				e_tep_event.key.time = key.time;
 				e_tep_event.key.state = key.state;
@@ -814,6 +851,7 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 #endif
 
 				_get_tep (edit);
+				edit->actions = 0;
 				return_val = e_text_event_processor_handle_event (edit->tep, &e_tep_event);
 				*actions = edit->actions;
 				if (e_tep_event.key.string) g_free (e_tep_event.key.string);
@@ -823,7 +861,9 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 
 		break;
 	case GDK_BUTTON_PRESS: /* Fall Through */
+		d(press = TRUE);
 	case GDK_BUTTON_RELEASE:
+		d(g_print ("%s: %s\n", __FUNCTION__, press ? "GDK_BUTTON_PRESS" : "GDK_BUTTON_RELEASE"));
 		event->button.x -= 4;
 		event->button.y -= 1;
 		if ((!edit_display) 
@@ -843,6 +883,7 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 			e_tep_event.button.button = button.button;
 			e_tep_event.button.position = _get_position_from_xy (cellptr, button.x, button.y);
 			_get_tep (edit);
+			edit->actions = 0;
 			return_val = e_text_event_processor_handle_event (edit->tep,
 									  &e_tep_event);
 			*actions = edit->actions;
@@ -865,6 +906,7 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 			e_tep_event.button.button = button.button;
 			e_tep_event.button.position = _get_position_from_xy (cellptr, button.x, button.y);
 			_get_tep (edit);
+			edit->actions = 0;
 			return_val = e_text_event_processor_handle_event (edit->tep,
 									  &e_tep_event);
 			*actions = edit->actions;
@@ -888,6 +930,7 @@ ect_event (ECellView *ecell_view, GdkEvent *event, int model_col, int view_col,
 			e_tep_event.motion.state = motion.state;
 			e_tep_event.motion.position = _get_position_from_xy (cellptr, motion.x, motion.y);
 			_get_tep (edit);
+			edit->actions = 0;
 			return_val = e_text_event_processor_handle_event (edit->tep,
 									  &e_tep_event);
 			*actions = edit->actions;
@@ -951,9 +994,9 @@ ect_height (ECellView *ecell_view, int model_col, int view_col, int row)
 	if (row == -1) {
 		value = e_font_height (font) + TEXT_PAD;
 	} else {
-		string = ect_get_text(ect, ecell_view->e_table_model, model_col, row);
+		string = e_cell_text_get_text(ect, ecell_view->e_table_model, model_col, row);
 		value = e_font_height (font) * number_of_lines(string) + TEXT_PAD;
-		ect_free_text(ect, string);
+		e_cell_text_free_text(ect, string);
 	}
 
 	return value;
@@ -1010,9 +1053,9 @@ ect_enter_edit (ECellView *ecell_view, int model_col, int view_col, int row)
 	edit->pointer_in = FALSE;
 	edit->default_cursor_shown = TRUE;
 	
-	temp = ect_get_text(ect, ecell_view->e_table_model, model_col, row);
+	temp = e_cell_text_get_text(ect, ecell_view->e_table_model, model_col, row);
 	edit->old_text = g_strdup (temp);
-	ect_free_text(ect, temp);
+	e_cell_text_free_text(ect, temp);
 	edit->cell.text = g_strdup (edit->old_text);
 
 #if 0
@@ -1039,8 +1082,7 @@ ect_leave_edit (ECellView *ecell_view, int model_col, int view_col, int row, voi
 	CellEdit *edit = text_view->edit;
 
 	if (edit){
-		ect_accept_edits (text_view);
-		ect_stop_editing (text_view);
+		ect_stop_editing (text_view, TRUE);
 		/* FIXME: edit is freed in ect_stop_editing() so I've
 		   commented this out - Damon. */
 		/*unbuild_current_cell (CURRENT_CELL(edit));*/
@@ -1059,7 +1101,7 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 	GnomeFont *font = gnome_font_new ("Helvetica", 12);
 	char *string;
 	ECellText *ect = E_CELL_TEXT(ecell_view->ecell);
-	string = ect_get_text(ect, ecell_view->e_table_model, model_col, row);
+	string = e_cell_text_get_text(ect, ecell_view->e_table_model, model_col, row);
 	gnome_print_gsave(context);
 	if (gnome_print_moveto(context, 2, 2) == -1)
 				/* FIXME */;
@@ -1077,7 +1119,7 @@ ect_print (ECellView *ecell_view, GnomePrintContext *context,
 	gnome_print_setfont(context, font);
 	gnome_print_show(context, string);
 	gnome_print_grestore(context);
-	ect_free_text(ect, string);
+	e_cell_text_free_text(ect, string);
 }
 
 static gdouble
@@ -1421,6 +1463,7 @@ e_cell_text_class_init (GtkObjectClass *object_class)
 
 	ectc->get_text = ect_real_get_text;
 	ectc->free_text = ect_real_free_text;
+	ectc->set_value = ect_real_set_value;
 
 	object_class->get_arg = ect_get_arg;
 	object_class->set_arg = ect_set_arg;
@@ -2022,7 +2065,7 @@ e_cell_text_view_command (ETextEventProcessor *tep, ETextEventProcessorCommand *
 		e_cell_text_view_get_selection (edit, GDK_SELECTION_PRIMARY, command->time);
 		break;
 	case E_TEP_ACTIVATE:
-		e_table_item_leave_edit (text_view->cell_view.e_table_item_view);
+		e_table_item_leave_edit_ (text_view->cell_view.e_table_item_view);
 		break;
 	case E_TEP_SET_SELECT_BY_WORD:
 		edit->select_by_word = command->value;
@@ -2415,9 +2458,9 @@ build_current_cell (CurrentCell *cell, ECellTextView *text_view, int model_col,
 	cell->row = row;
 	cell->breaks = NULL;
 
-	temp = ect_get_text(ect, ecell_view->e_table_model, model_col, row);
+	temp = e_cell_text_get_text(ect, ecell_view->e_table_model, model_col, row);
 	cell->text = g_strdup(temp);
-	ect_free_text(ect, temp);
+	e_cell_text_free_text(ect, temp);
 
 	cell->width = e_table_header_get_column (
 		((ETableItem *)ecell_view->e_table_item_view)->header,
