@@ -1,12 +1,26 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-table-extras.c: Pair of hash table sort of thingies.
+/* 
+ * e-table-extras.c - Set of hash table sort of thingies.
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
+ * Authors:
  *   Chris Lahey <clahey@ximian.com>
  *
- * (C) 2000 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <stdlib.h>
 #include <gtk/gtksignal.h>
@@ -15,6 +29,7 @@
 #include "gal/e-table/e-cell-checkbox.h"
 #include "gal/e-table/e-cell-date.h"
 #include "gal/e-table/e-cell-number.h"
+#include "gal/e-table/e-cell-pixbuf.h"
 #include "gal/e-table/e-cell-size.h"
 #include "gal/e-table/e-cell-tree.h"
 #include "e-table-extras.h"
@@ -71,6 +86,15 @@ ete_class_init (GtkObjectClass *klass)
 	klass->destroy = ete_destroy;
 }
 
+static gint
+e_strint_compare(gconstpointer data1, gconstpointer data2)
+{
+	int int1 = atoi(data1);
+	int int2 = atoi(data2);
+
+	return g_int_compare(GINT_TO_POINTER(int1), GINT_TO_POINTER(int2));
+}
+
 static void
 ete_init (ETableExtras *extras)
 {
@@ -80,10 +104,12 @@ ete_init (ETableExtras *extras)
 
 	e_table_extras_add_compare(extras, "string", g_str_compare);
 	e_table_extras_add_compare(extras, "integer", g_int_compare);
+	e_table_extras_add_compare(extras, "string-integer", e_strint_compare);
 
 	e_table_extras_add_cell(extras, "checkbox", e_cell_checkbox_new());
 	e_table_extras_add_cell(extras, "date", e_cell_date_new (NULL, GTK_JUSTIFY_LEFT));
 	e_table_extras_add_cell(extras, "number", e_cell_number_new (NULL, GTK_JUSTIFY_RIGHT));
+	e_table_extras_add_cell(extras, "pixbuf", e_cell_pixbuf_new ());
 	e_table_extras_add_cell(extras, "size", e_cell_size_new (NULL, GTK_JUSTIFY_RIGHT));
 	e_table_extras_add_cell(extras, "string", e_cell_text_new (NULL, GTK_JUSTIFY_LEFT));
 	e_table_extras_add_cell(extras, "tree-string", e_cell_tree_new (NULL, NULL, TRUE, e_cell_text_new (NULL, GTK_JUSTIFY_LEFT)));
@@ -108,6 +134,7 @@ e_table_extras_add_cell     (ETableExtras *extras,
 	ECell *old_cell;
 
 	if (g_hash_table_lookup_extended (extras->cells, id, (gpointer *)&old_key, (gpointer *)&old_cell)) {
+		g_hash_table_remove (extras->cells, old_key);
 		g_free (old_key);
 		if (old_cell)
 			gtk_object_unref (GTK_OBJECT(old_cell));
@@ -136,6 +163,7 @@ e_table_extras_add_compare  (ETableExtras *extras,
 	GCompareFunc old_compare;
 
 	if (g_hash_table_lookup_extended (extras->cells, id, (gpointer *)&old_key, (gpointer *)&old_compare)) {
+		g_hash_table_remove (extras->cells, old_key);
 		g_free (old_key);
 	}
 
@@ -158,6 +186,7 @@ e_table_extras_add_pixbuf     (ETableExtras *extras,
 	GdkPixbuf *old_pixbuf;
 
 	if (g_hash_table_lookup_extended (extras->pixbufs, id, (gpointer *)&old_key, (gpointer *)&old_pixbuf)) {
+		g_hash_table_remove (extras->cells, old_key);
 		g_free (old_key);
 		if (old_pixbuf)
 			gdk_pixbuf_unref (old_pixbuf);
