@@ -1,12 +1,24 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-Table-Group.c: Implements the grouping objects for elements on a table
+/* 
+ * e-table-group.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@ximian.com)
- *   Chris Lahey (clahey@ximian.com)
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
- * Copyright 1999, 2000 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
@@ -44,17 +56,27 @@ static void
 etg_destroy (GtkObject *object)
 {
 	ETableGroup *etg = E_TABLE_GROUP(object);
-	if (etg->header)
+
+	if (etg->header) {
 		gtk_object_unref (GTK_OBJECT(etg->header));
-	if (etg->full_header)
+		etg->header = NULL;
+	}
+
+	if (etg->full_header) {
 		gtk_object_unref (GTK_OBJECT(etg->full_header));
-	if (etg->model)
+		etg->full_header = NULL;
+	}
+
+	if (etg->model) {
 		gtk_object_unref (GTK_OBJECT(etg->model));
+		etg->model = NULL;
+	}
+
 	if (GTK_OBJECT_CLASS (etg_parent_class)->destroy)
 		GTK_OBJECT_CLASS (etg_parent_class)->destroy (object);
 }
 
-/** 
+/**
  * e_table_group_new
  * @parent: The %GnomeCanvasGroup to create a child of.
  * @full_header: The full header of the %ETable.
@@ -81,7 +103,7 @@ e_table_group_new (GnomeCanvasGroup *parent,
 		   int               n)
 {
 	g_return_val_if_fail (model != NULL, NULL);
-	
+
 	if (n < e_table_sort_info_grouping_get_count(sort_info)) {
 		return e_table_group_container_new (parent, full_header, header, model, sort_info, n);
 	} else {
@@ -90,7 +112,7 @@ e_table_group_new (GnomeCanvasGroup *parent,
 	return NULL;
 }
 
-/** 
+/**
  * e_table_group_construct
  * @parent: The %GnomeCanvasGroup to create a child of.
  * @etg: The %ETableGroup to construct.
@@ -116,7 +138,7 @@ e_table_group_construct (GnomeCanvasGroup *parent,
 	gnome_canvas_item_constructv (GNOME_CANVAS_ITEM (etg), parent, 0, NULL);
 }
 
-/** 
+/**
  * e_table_group_add
  * @etg: The %ETableGroup to add a row to
  * @row: The row to add.
@@ -131,11 +153,11 @@ e_table_group_add (ETableGroup *etg,
 	g_return_if_fail (etg != NULL);
 	g_return_if_fail (E_IS_TABLE_GROUP (etg));
 
-	if (ETG_CLASS (etg)->add)
-		ETG_CLASS (etg)->add (etg, row);
+	g_assert (ETG_CLASS (etg)->add != NULL);
+	ETG_CLASS (etg)->add (etg, row);
 }
 
-/** 
+/**
  * e_table_group_add_array
  * @etg: The %ETableGroup to add to
  * @array: The array to add.
@@ -152,11 +174,11 @@ e_table_group_add_array (ETableGroup *etg,
 	g_return_if_fail (etg != NULL);
 	g_return_if_fail (E_IS_TABLE_GROUP (etg));
 
-	if (ETG_CLASS (etg)->add_array)
-		ETG_CLASS (etg)->add_array (etg, array, count);
+	g_assert (ETG_CLASS (etg)->add_array != NULL);
+	ETG_CLASS (etg)->add_array (etg, array, count);
 }
 
-/** 
+/**
  * e_table_group_add_all
  * @etg: The %ETableGroup to add to
  *
@@ -169,11 +191,11 @@ e_table_group_add_all (ETableGroup *etg)
 	g_return_if_fail (etg != NULL);
 	g_return_if_fail (E_IS_TABLE_GROUP (etg));
 
-	if (ETG_CLASS (etg)->add_all)
-		ETG_CLASS (etg)->add_all (etg);
+	g_assert (ETG_CLASS (etg)->add_all != NULL);
+	ETG_CLASS (etg)->add_all (etg);
 }
 
-/** 
+/**
  * e_table_group_remove
  * @etg: The %ETableGroup to remove a row from
  * @row: The row to remove.
@@ -191,13 +213,11 @@ e_table_group_remove (ETableGroup *etg,
 	g_return_val_if_fail (etg != NULL, FALSE);
 	g_return_val_if_fail (E_IS_TABLE_GROUP (etg), FALSE);
 
-	if (ETG_CLASS (etg)->remove)
-		return ETG_CLASS (etg)->remove (etg, row);
-	else
-		return FALSE;
+	g_assert (ETG_CLASS (etg)->remove != NULL);
+	return ETG_CLASS (etg)->remove (etg, row);
 }
 
-/** 
+/**
  * e_table_group_increment
  * @etg: The %ETableGroup to increment
  * @position: The position to increment from
@@ -215,11 +235,11 @@ e_table_group_increment (ETableGroup *etg,
 	g_return_if_fail (etg != NULL);
 	g_return_if_fail (E_IS_TABLE_GROUP (etg));
 
-	if (ETG_CLASS (etg)->increment)
-		ETG_CLASS (etg)->increment (etg, position, amount);
+	g_assert (ETG_CLASS (etg)->increment != NULL);
+	ETG_CLASS (etg)->increment (etg, position, amount);
 }
 
-/** 
+/**
  * e_table_group_increment
  * @etg: The %ETableGroup to decrement
  * @position: The position to decrement from
@@ -237,11 +257,11 @@ e_table_group_decrement (ETableGroup *etg,
 	g_return_if_fail (etg != NULL);
 	g_return_if_fail (E_IS_TABLE_GROUP (etg));
 
-	if (ETG_CLASS (etg)->decrement)
-		ETG_CLASS (etg)->decrement (etg, position, amount);
+	g_assert (ETG_CLASS (etg)->decrement != NULL);
+	ETG_CLASS (etg)->decrement (etg, position, amount);
 }
 
-/** 
+/**
  * e_table_group_increment
  * @etg: The %ETableGroup to count
  *
@@ -253,15 +273,13 @@ gint
 e_table_group_row_count (ETableGroup *etg)
 {
 	g_return_val_if_fail (etg != NULL, 0);
-	g_return_val_if_fail (E_IS_TABLE_GROUP (etg), 0);
+	g_return_val_if_fail (E_IS_TABLE_GROUP (etg), -1);
 
-	if (ETG_CLASS (etg)->row_count)
-		return ETG_CLASS (etg)->row_count (etg);
-	else
-		return 0;
+	g_assert (ETG_CLASS (etg)->row_count != NULL);
+	return ETG_CLASS (etg)->row_count (etg);
 }
 
-/** 
+/**
  * e_table_group_set_focus
  * @etg: The %ETableGroup to set
  * @direction: The direction the focus is coming from.
@@ -278,11 +296,11 @@ e_table_group_set_focus (ETableGroup *etg,
 	g_return_if_fail (etg != NULL);
 	g_return_if_fail (E_IS_TABLE_GROUP (etg));
 
-	if (ETG_CLASS (etg)->set_focus)
-		ETG_CLASS (etg)->set_focus (etg, direction, view_col);
+	g_assert (ETG_CLASS (etg)->set_focus != NULL);
+	ETG_CLASS (etg)->set_focus (etg, direction, view_col);
 }
 
-/** 
+/**
  * e_table_group_get_focus
  * @etg: The %ETableGroup to check
  *
@@ -296,13 +314,11 @@ e_table_group_get_focus (ETableGroup *etg)
 	g_return_val_if_fail (etg != NULL, FALSE);
 	g_return_val_if_fail (E_IS_TABLE_GROUP (etg), FALSE);
 
-	if (ETG_CLASS (etg)->get_focus)
-		return ETG_CLASS (etg)->get_focus (etg);
-	else
-		return FALSE;
+	g_assert (ETG_CLASS (etg)->get_focus != NULL);
+	return ETG_CLASS (etg)->get_focus (etg);
 }
 
-/** 
+/**
  * e_table_group_get_focus_column
  * @etg: The %ETableGroup to check
  *
@@ -314,15 +330,13 @@ gint
 e_table_group_get_focus_column (ETableGroup *etg)
 {
 	g_return_val_if_fail (etg != NULL, FALSE);
-	g_return_val_if_fail (E_IS_TABLE_GROUP (etg), FALSE);
+	g_return_val_if_fail (E_IS_TABLE_GROUP (etg), -1);
 
-	if (ETG_CLASS (etg)->get_focus_column)
-		return ETG_CLASS (etg)->get_focus_column (etg);
-	else
-		return -1;
+	g_assert (ETG_CLASS (etg)->get_focus_column != NULL);
+	return ETG_CLASS (etg)->get_focus_column (etg);
 }
 
-/** 
+/**
  * e_table_group_get_printable
  * @etg: %ETableGroup which will be printed
  *
@@ -337,13 +351,11 @@ e_table_group_get_printable (ETableGroup *etg)
 	g_return_val_if_fail (etg != NULL, NULL);
 	g_return_val_if_fail (E_IS_TABLE_GROUP (etg), NULL);
 
-	if (ETG_CLASS (etg)->get_printable)
-		return ETG_CLASS (etg)->get_printable (etg);
-	else
-		return NULL;
+	g_assert (ETG_CLASS (etg)->get_printable != NULL);
+	return ETG_CLASS (etg)->get_printable (etg);
 }
 
-/** 
+/**
  * e_table_group_compute_location
  * @eti: %ETableGroup to look in.
  * @x: A pointer to the x location to find in the %ETableGroup.
@@ -363,11 +375,11 @@ e_table_group_compute_location (ETableGroup *etg, int *x, int *y, int *row, int
 	g_return_if_fail (etg != NULL);
 	g_return_if_fail (E_IS_TABLE_GROUP (etg));
 
-	if (ETG_CLASS (etg)->compute_location)
-		ETG_CLASS (etg)->compute_location (etg, x, y, row, col);
+	g_assert (ETG_CLASS (etg)->compute_location != NULL);
+	ETG_CLASS (etg)->compute_location (etg, x, y, row, col);
 }
 
-/** 
+/**
  * e_table_group_get_position
  * @eti: %ETableGroup to look in.
  * @x: A pointer to the location to store the found x location in.
@@ -375,10 +387,10 @@ e_table_group_compute_location (ETableGroup *etg, int *x, int *y, int *row, int
  * @row: A pointer to the row number to find.
  * @col: A pointer to the col number to find.
  *
- * This routine finds the view cell (row, col) in the %ETableGroup.
- * If that location is in the %ETableGroup *x and *y are set to the
+ * This routine finds the view cell (row, col) in the #ETableGroup.
+ * If that location is in the #ETableGroup *@x and *@y are set to the
  * upper left hand corner of the cell found.  If that location is not
- * in the %ETableGroup, the number of rows in the %ETableGroup is
+ * in the #ETableGroup, the number of rows in the #ETableGroup is
  * removed from the value row points to.
  */
 void
@@ -393,11 +405,11 @@ e_table_group_get_cell_geometry  (ETableGroup *etg,
 	g_return_if_fail (etg != NULL);
 	g_return_if_fail (E_IS_TABLE_GROUP (etg));
 
-	if (ETG_CLASS (etg)->get_cell_geometry)
-		ETG_CLASS (etg)->get_cell_geometry (etg, row, col, x, y, width, height);
+	g_assert (ETG_CLASS (etg)->get_cell_geometry != NULL);
+	ETG_CLASS (etg)->get_cell_geometry (etg, row, col, x, y, width, height);
 }
 
-/** 
+/**
  * e_table_group_cursor_change
  * @eti: %ETableGroup to emit the signal on
  * @row: The new cursor row (model row)
@@ -415,7 +427,7 @@ e_table_group_cursor_change (ETableGroup *e_table_group, gint row)
 			 row);
 }
 
-/** 
+/**
  * e_table_group_cursor_activated
  * @eti: %ETableGroup to emit the signal on
  * @row: The cursor row (model row)
@@ -433,7 +445,7 @@ e_table_group_cursor_activated (ETableGroup *e_table_group, gint row)
 			 row);
 }
 
-/** 
+/**
  * e_table_group_double_click
  * @eti: %ETableGroup to emit the signal on
  * @row: The row clicked on (model row)
@@ -453,7 +465,7 @@ e_table_group_double_click (ETableGroup *e_table_group, gint row, gint col, GdkE
 			 row, col, event);
 }
 
-/** 
+/**
  * e_table_group_right_click
  * @eti: %ETableGroup to emit the signal on
  * @row: The row clicked on (model row)
@@ -477,7 +489,7 @@ e_table_group_right_click (ETableGroup *e_table_group, gint row, gint col, GdkEv
 	return return_val;
 }
 
-/** 
+/**
  * e_table_group_click
  * @eti: %ETableGroup to emit the signal on
  * @row: The row clicked on (model row)
@@ -501,7 +513,7 @@ e_table_group_click (ETableGroup *e_table_group, gint row, gint col, GdkEvent *e
 	return return_val;
 }
 
-/** 
+/**
  * e_table_group_key_press
  * @eti: %ETableGroup to emit the signal on
  * @row: The cursor row (model row)
@@ -525,7 +537,7 @@ e_table_group_key_press (ETableGroup *e_table_group, gint row, gint col, GdkEven
 	return return_val;
 }
 
-/** 
+/**
  * e_table_group_start_drag
  * @eti: %ETableGroup to emit the signal on
  * @row: The cursor row (model row)
@@ -549,7 +561,7 @@ e_table_group_start_drag (ETableGroup *e_table_group, gint row, gint col, GdkEve
 	return return_val;
 }
 
-/** 
+/**
  * e_table_group_get_header
  * @eti: %ETableGroup to check
  *
@@ -612,7 +624,7 @@ etg_class_init (GtkObjectClass *object_class)
 	klass->click = NULL;
 	klass->key_press = NULL;
 	klass->start_drag = NULL;
-	
+
 	klass->add = NULL;
 	klass->add_array = NULL;
 	klass->add_all = NULL;
@@ -631,7 +643,7 @@ etg_class_init (GtkObjectClass *object_class)
 	etg_signals [CURSOR_CHANGE] =
 		gtk_signal_new ("cursor_change",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableGroupClass, cursor_change),
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
@@ -639,7 +651,7 @@ etg_class_init (GtkObjectClass *object_class)
 	etg_signals [CURSOR_ACTIVATED] =
 		gtk_signal_new ("cursor_activated",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableGroupClass, cursor_activated),
 				gtk_marshal_NONE__INT,
 				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
@@ -647,7 +659,7 @@ etg_class_init (GtkObjectClass *object_class)
 	etg_signals [DOUBLE_CLICK] =
 		gtk_signal_new ("double_click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableGroupClass, double_click),
 				gtk_marshal_NONE__INT_INT_POINTER,
 				GTK_TYPE_NONE, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -655,7 +667,7 @@ etg_class_init (GtkObjectClass *object_class)
 	etg_signals [RIGHT_CLICK] =
 		gtk_signal_new ("right_click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableGroupClass, right_click),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -663,7 +675,7 @@ etg_class_init (GtkObjectClass *object_class)
 	etg_signals [CLICK] =
 		gtk_signal_new ("click",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableGroupClass, click),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -671,7 +683,7 @@ etg_class_init (GtkObjectClass *object_class)
 	etg_signals [KEY_PRESS] =
 		gtk_signal_new ("key_press",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableGroupClass, key_press),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
@@ -679,12 +691,12 @@ etg_class_init (GtkObjectClass *object_class)
 	etg_signals [START_DRAG] =
 		gtk_signal_new ("start_drag",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableGroupClass, start_drag),
 				e_marshal_INT__INT_INT_POINTER,
 				GTK_TYPE_INT, 3, GTK_TYPE_INT, GTK_TYPE_INT, GTK_TYPE_GDK_EVENT);
 
-	gtk_object_class_add_signals (object_class, etg_signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, etg_signals, LAST_SIGNAL);
 }
 
 E_MAKE_TYPE (e_table_group, "ETableGroup", ETableGroup, etg_class_init, NULL, PARENT_TYPE);
