@@ -1,13 +1,26 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * E-table-scrolled.c: A graphical view of a Table.
+/* 
+ * e-table-scrolled.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
+ * Authors:
  *   Chris Lahey <clahey@ximian.com>
- *   Miguel de Icaza (miguel@ximian.com)
  *
- * Copyright 2000, 1999, Ximian, Inc
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -15,8 +28,8 @@
 #include <stdio.h>
 #include <libgnomeui/gnome-canvas.h>
 #include <gtk/gtksignal.h>
-#include <gnome-xml/parser.h>
-#include <gnome-xml/xmlmemory.h>
+#include <parser.h>
+#include <xmlmemory.h>
 
 #include "e-table.h"
 #include "e-table-scrolled.h"
@@ -52,12 +65,6 @@ e_table_scrolled_init (GtkObject *object)
 static void
 e_table_scrolled_real_construct (ETableScrolled *ets)
 {
-	gtk_object_set(GTK_OBJECT(ets),
-		       "shadow_type", GTK_SHADOW_IN,
-		       "hscrollbar_policy", GTK_POLICY_NEVER,
-		       "vscrollbar_policy", GTK_POLICY_AUTOMATIC,
-		       NULL);
-
 	gtk_container_add(GTK_CONTAINER(ets), GTK_WIDGET(ets->table));
 
 	gtk_widget_show(GTK_WIDGET(ets->table));
@@ -137,7 +144,10 @@ GtkWidget      *e_table_scrolled_new_from_spec_file        (ETableModel       *e
 	g_return_val_if_fail(ete == NULL || E_IS_TABLE_EXTRAS(ete), NULL);
 	g_return_val_if_fail(spec_fn != NULL, NULL);
 
-	ets = gtk_type_new (e_table_scrolled_get_type ());
+	ets = E_TABLE_SCROLLED (gtk_widget_new (e_table_scrolled_get_type (),
+						"hadjustment", NULL,
+						"vadjustment", NULL,
+						NULL));
 
 	ets = e_table_scrolled_construct_from_spec_file (ets, etm, ete, spec_fn, state_fn);
 
