@@ -1,13 +1,26 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * e-table-sort-info.c: a Table Model
+/* 
+ * e-table-sort-info.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Miguel de Icaza (miguel@gnu.org)
- *   Chris Lahey (clahey@ximian.com)
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
  *
- * (C) 1999, 2000, 2001 Ximian, Inc.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 
 #include "e-table-sort-info.h"
@@ -76,7 +89,7 @@ e_table_sort_info_class_init (ETableSortInfoClass *klass)
 	e_table_sort_info_signals [SORT_INFO_CHANGED] =
 		gtk_signal_new ("sort_info_changed",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableSortInfoClass, sort_info_changed),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -84,7 +97,7 @@ e_table_sort_info_class_init (ETableSortInfoClass *klass)
 	e_table_sort_info_signals [GROUP_INFO_CHANGED] =
 		gtk_signal_new ("group_info_changed",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ETableSortInfoClass, group_info_changed),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -92,7 +105,7 @@ e_table_sort_info_class_init (ETableSortInfoClass *klass)
 	klass->sort_info_changed = NULL;
 	klass->group_info_changed = NULL;
 
-	gtk_object_class_add_signals (object_class, e_table_sort_info_signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_table_sort_info_signals, LAST_SIGNAL);
 }
 
 E_MAKE_TYPE(e_table_sort_info, "ETableSortInfo", ETableSortInfo,
@@ -362,14 +375,14 @@ e_table_sort_info_load_from_node (ETableSortInfo *info,
 
 	if (state_version <= 0.05) {
 		i = 0;
-		for (grouping = node->childs; grouping && !strcmp (grouping->name, "group"); grouping = grouping->childs) {
+		for (grouping = node->xmlChildrenNode; grouping && !strcmp (grouping->name, "group"); grouping = grouping->xmlChildrenNode) {
 			ETableSortColumn column;
 			column.column = e_xml_get_integer_prop_by_name (grouping, "column");
 			column.ascending = e_xml_get_bool_prop_by_name (grouping, "ascending");
 			e_table_sort_info_grouping_set_nth(info, i++, column);
 		}
 		i = 0;
-		for (; grouping && !strcmp (grouping->name, "leaf"); grouping = grouping->childs) {
+		for (; grouping && !strcmp (grouping->name, "leaf"); grouping = grouping->xmlChildrenNode) {
 			ETableSortColumn column;
 			column.column = e_xml_get_integer_prop_by_name (grouping, "column");
 			column.ascending = e_xml_get_bool_prop_by_name (grouping, "ascending");
@@ -377,7 +390,7 @@ e_table_sort_info_load_from_node (ETableSortInfo *info,
 		}
 	} else {
 		i = 0;
-		for (grouping = node->childs; grouping && !strcmp (grouping->name, "group"); grouping = grouping->next) {
+		for (grouping = node->xmlChildrenNode; grouping && !strcmp (grouping->name, "group"); grouping = grouping->next) {
 			ETableSortColumn column;
 			column.column = e_xml_get_integer_prop_by_name (grouping, "column");
 			column.ascending = e_xml_get_bool_prop_by_name (grouping, "ascending");
