@@ -1,15 +1,27 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/*
- * e-tree-model.c: a Tree Model
+/* 
+ * e-tree-model.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author:
- *   Chris Toshok (toshok@ximian.com)
- *   Chris Lahey  <clahey@ximian.com>
+ * Authors:
+ *   Chris Lahey <clahey@ximian.com>
+ *   Chris Toshok <toshok@ximian.com>
  *
- * Adapted from the gtree code and ETableModel.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * (C) 2000, 2001 Ximian, Inc.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
+
 #include <config.h>
 
 #include <stdio.h>
@@ -18,8 +30,8 @@
 #include <unistd.h>
 #include <fcntl.h>
 
-#include <gnome-xml/parser.h>
-#include <gnome-xml/xmlmemory.h>
+#include <parser.h>
+#include <xmlmemory.h>
 
 #include <gtk/gtksignal.h>
 #include "gal/util/e-util.h"
@@ -34,11 +46,13 @@ static GtkObjectClass *parent_class;
 
 enum {
 	PRE_CHANGE,
+	NO_CHANGE,
 	NODE_CHANGED,
 	NODE_DATA_CHANGED,
 	NODE_COL_CHANGED,
 	NODE_INSERTED,
 	NODE_REMOVED,
+	NODE_DELETED,
 	LAST_SIGNAL
 };
 
@@ -60,6 +74,14 @@ e_tree_model_class_init (GtkObjectClass *klass)
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
 
+	e_tree_model_signals [NO_CHANGE] =
+		gtk_signal_new ("no_change",
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (klass),
+				GTK_SIGNAL_OFFSET (ETreeModelClass, no_change),
+				gtk_marshal_NONE__NONE,
+				GTK_TYPE_NONE, 0);
+
 	e_tree_model_signals [NODE_CHANGED] =
 		gtk_signal_new ("node_changed",
 				GTK_RUN_LAST,
@@ -100,6 +122,14 @@ e_tree_model_class_init (GtkObjectClass *klass)
 				e_marshal_NONE__POINTER_POINTER_INT,
 				GTK_TYPE_NONE, 3, GTK_TYPE_POINTER, GTK_TYPE_POINTER, GTK_TYPE_INT);
 
+	e_tree_model_signals [NODE_DELETED] =
+		gtk_signal_new ("node_deleted",
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (klass),
+				GTK_SIGNAL_OFFSET (ETreeModelClass, node_deleted),
+				gtk_marshal_NONE__POINTER,
+				GTK_TYPE_NONE, 1, GTK_TYPE_POINTER);
+
 	E_OBJECT_CLASS_ADD_SIGNALS (klass, e_tree_model_signals, LAST_SIGNAL);
 
 	tree_class->get_root             = NULL;
@@ -138,11 +168,13 @@ e_tree_model_class_init (GtkObjectClass *klass)
 	tree_class->value_to_string      = NULL;
 
 	tree_class->pre_change           = NULL;
+	tree_class->no_change            = NULL;
 	tree_class->node_changed         = NULL;
 	tree_class->node_data_changed    = NULL;
 	tree_class->node_col_changed     = NULL;
 	tree_class->node_inserted        = NULL;
 	tree_class->node_removed         = NULL;
+	tree_class->node_deleted         = NULL;
 }
 
 static void
@@ -174,6 +206,25 @@ e_tree_model_pre_change  (ETreeModel *tree_model)
 			 e_tree_model_signals [PRE_CHANGE]);
 }
 
+/**
+ * e_tree_model_node_changed:
+ * @tree_model: 
+ * @node: 
+ * 
+ * 
+ * 
+ * Return value: 
+ **/
+void
+e_tree_model_no_change  (ETreeModel *tree_model)
+{
+	g_return_if_fail (tree_model != NULL);
+	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
+	
+	gtk_signal_emit (GTK_OBJECT (tree_model),
+			 e_tree_model_signals [NO_CHANGE]);
+}
+
 /**
  * e_tree_model_node_changed:
  * @tree_model: 
@@ -271,6 +322,24 @@ e_tree_model_node_removed  (ETreeModel *tree_model, ETreePath parent_node, ETree
 			 parent_node, removed_node, old_position);
 }
 
+/**
+ * e_tree_model_node_deleted:
+ * @tree_model: 
+ * @deleted_node: 
+ * 
+ * 
+ **/
+void
+e_tree_model_node_deleted  (ETreeModel *tree_model, ETreePath deleted_node)
+{
+	g_return_if_fail (tree_model != NULL);
+	g_return_if_fail (E_IS_TREE_MODEL (tree_model));
+	
+	gtk_signal_emit (GTK_OBJECT (tree_model),
+			 e_tree_model_signals [NODE_DELETED],
+			 deleted_node);
+}
+
 
 
 /**
