@@ -1,31 +1,27 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-
-/*
- * ECompletionView - A text completion selection widget
- * Copyright (C) 2000, 2001 Ximian, Inc.
+/* 
+ * e-completion-view.c - A text completion selection widget
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * Author: Jon Trowbridge <trow@ximian.com>
- * Adapted from code by Miguel de Icaza <miguel@ximian.com>
+ * Authors:
+ *   Miguel de Icaza <miguel@ximian.com>
+ *   Adapted by Jon Trowbridge <trow@ximian.com>
+ *
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
-/*
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
- */
-
-
 #include <config.h>
 #include <math.h>
 #include <gdk/gdkkeysyms.h>
@@ -225,7 +221,7 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 	e_completion_view_signals[E_COMPLETION_VIEW_NONEMPTY] =
 		gtk_signal_new ("nonempty",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, nonempty),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -233,7 +229,7 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 	e_completion_view_signals[E_COMPLETION_VIEW_ADDED] =
 		gtk_signal_new ("added",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, added),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -241,7 +237,7 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 	e_completion_view_signals[E_COMPLETION_VIEW_FULL] =
 		gtk_signal_new ("full",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, full),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -249,7 +245,7 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 	e_completion_view_signals[E_COMPLETION_VIEW_BROWSE] =
 		gtk_signal_new ("browse",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, browse),
 				gtk_marshal_NONE__POINTER,
 				GTK_TYPE_NONE, 1,
@@ -258,7 +254,7 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 	e_completion_view_signals[E_COMPLETION_VIEW_UNBROWSE] =
 		gtk_signal_new ("unbrowse",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, unbrowse),
 				gtk_marshal_NONE__NONE,
 				GTK_TYPE_NONE, 0);
@@ -266,13 +262,13 @@ e_completion_view_class_init (ECompletionViewClass *klass)
 	e_completion_view_signals[E_COMPLETION_VIEW_ACTIVATE] =
 		gtk_signal_new ("activate",
 				GTK_RUN_LAST,
-				object_class->type,
+				E_OBJECT_CLASS_TYPE (object_class),
 				GTK_SIGNAL_OFFSET (ECompletionViewClass, activate),
 				gtk_marshal_NONE__POINTER,
 				GTK_TYPE_NONE, 1,
 				GTK_TYPE_POINTER);
 
-	gtk_object_class_add_signals (object_class, e_completion_view_signals, E_COMPLETION_VIEW_LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_completion_view_signals, E_COMPLETION_VIEW_LAST_SIGNAL);
 
 	object_class->destroy = e_completion_view_destroy;
 
