@@ -1,31 +1,25 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-
-/*
- * EEntry: An EText-based entry widget
+/* 
+ * e-entry.c - An EText-based entry widget
+ * Copyright 2000, 2001, Ximian, Inc.
  *
  * Authors:
- *   Miguel de Icaza <miguel@helixcode.com>
- *   Chris Lahey     <clahey@helixcode.com>
+ *   Chris Lahey     <clahey@ximian.com>
  *   Jon Trowbridge  <trow@ximian.com>
  *
- * Copyright (C) 1999, 2000, 2001 Ximian Inc.
- */
-
-/*
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
@@ -49,8 +43,17 @@
 #include "e-text.h"
 #include "e-entry.h"
 
+#define MOVE_RIGHT_AND_UP 0
+
+#define EVIL_POINTER_WARPING_HACK
+
+#ifdef EVIL_POINTER_WARPING_HACK
+#include <gdk/gdkx.h>
+#endif
+
 #define MIN_ENTRY_WIDTH  150
-#define INNER_BORDER     2
+
+#define d(x)
 
 #define PARENT_TYPE gtk_table_get_type ()
 
@@ -60,6 +63,7 @@ enum {
 	E_ENTRY_CHANGED,
 	E_ENTRY_ACTIVATE,
 	E_ENTRY_POPUP,
+	E_ENTRY_COMPLETION_POPUP,
 	E_ENTRY_LAST_SIGNAL
 };
 
@@ -91,7 +95,9 @@ enum {
 	ARG_ALLOW_NEWLINES,
 	ARG_DRAW_BORDERS,
 	ARG_DRAW_BACKGROUND,
-	ARG_CURSOR_POS
+	ARG_DRAW_BUTTON,
+	ARG_EMULATE_LABEL_RESIZE,
+	ARG_CURSOR_POS,
 };
 
 typedef struct _EEntryPrivate EEntryPrivate;
@@ -101,7 +107,6 @@ struct _EEntryPrivate {
 	guint changed_proxy_tag;
 	guint activate_proxy_tag;
 	guint popup_proxy_tag;
-
 	/* Data related to completions */
 	ECompletion *completion;
 	EEntryCompletionHandler handler;
@@ -123,6 +128,9 @@ struct _EEntryPrivate {
 	gint last_completion_pos;
 
 	guint draw_borders : 1;
+	guint emulate_label_resize : 1;
+	guint have_set_transient : 1;
+	gint last_width;
 };
 
 static gboolean e_entry_is_empty              (EEntry *entry);
@@ -135,8 +143,6 @@ static void
 canvas_size_allocate (GtkWidget *widget, GtkAllocation *alloc,
 		      EEntry *entry)
 {
-	gint xthick;
-	gint ythick;
 	gnome_canvas_set_scroll_region (entry->canvas,
 					0, 0, alloc->width, alloc->height);
 	gtk_object_set (GTK_OBJECT (entry->item),
@@ -144,26 +150,18 @@ canvas_size_allocate (GtkWidget *widget, GtkAllocation *alloc,
 			"clip_height", (double) (alloc->height),
 			NULL);
 
-	if (entry->priv->draw_borders) {
-		xthick = 0;
-		ythick = 0;
-	} else {
-		xthick = widget->style->klass->xthickness;
-		ythick = widget->style->klass->ythickness;
-	}
-
 	switch (entry->priv->justification) {
 	case GTK_JUSTIFY_RIGHT:
 		e_canvas_item_move_absolute(GNOME_CANVAS_ITEM(entry->item),
-					    alloc->width - xthick, ythick);
+					    alloc->width, 0);
 		break;
 	case GTK_JUSTIFY_CENTER:
 		e_canvas_item_move_absolute(GNOME_CANVAS_ITEM(entry->item),
-					    alloc->width / 2, ythick);
+					    alloc->width / 2, 0);
 		break;
 	default:
 		e_canvas_item_move_absolute(GNOME_CANVAS_ITEM(entry->item),
-					    xthick, ythick);
+					    0, 0);
 		break;
 	}
 }
@@ -172,21 +170,37 @@ static void
 canvas_size_request (GtkWidget *widget, GtkRequisition *requisition,
 		     EEntry *entry)
 {
-	int border;
+	int xthick, ythick;
 	
 	g_return_if_fail (widget != NULL);
 	g_return_if_fail (GNOME_IS_CANVAS (widget));
 	g_return_if_fail (requisition != NULL);
 
-	if (entry->priv->draw_borders)
-		border = INNER_BORDER;
-	else
-		border = 0;
-	
-	requisition->width = MIN_ENTRY_WIDTH + (widget->style->klass->xthickness + border) * 2;
-	requisition->height = (widget->style->font->ascent +
+	if (entry->priv->draw_borders) {
+		xthick = 2 * widget->style->klass->xthickness;
+		ythick = 2 * widget->style->klass->ythickness;
+	} else {
+		xthick = ythick = 0;
+	}
+
+	if (entry->priv->emulate_label_resize) {
+		gdouble width;
+		gtk_object_get (GTK_OBJECT (entry->item),
+				"text_width", &width,
+				NULL);
+		requisition->width = 2 + xthick + width;
+	} else {
+		requisition->width = 2 + MIN_ENTRY_WIDTH + xthick;
+	}
+	if (entry->priv->last_width != requisition->width)
+		gtk_widget_queue_resize (widget);
+	entry->priv->last_width = requisition->width;
+
+	d(g_print("%s: width = %d\n", __FUNCTION__, requisition->width));
+
+	requisition->height = (2 + widget->style->font->ascent +
 			       widget->style->font->descent +
-			       (widget->style->klass->ythickness + border) * 2);
+			       ythick);
 }
 
 static gint
@@ -195,7 +209,7 @@ canvas_focus_in_event (GtkWidget *widget, GdkEventFocus *focus, EEntry *entry)
 	if (entry->canvas->focused_item != GNOME_CANVAS_ITEM(entry->item))
 		gnome_canvas_item_grab_focus(GNOME_CANVAS_ITEM(entry->item));
 
-	return 0;
+	return FALSE;
 }
 
 static void
@@ -211,10 +225,13 @@ e_entry_text_keypress (EText *text, guint keyval, guint state, EEntry *entry)
 		if (e_entry_is_empty (entry)) {
 			e_entry_cancel_delayed_completion (entry);
 			e_entry_show_popup (entry, FALSE);
-		} else if (entry->priv->popup_is_visible) {
-			e_entry_start_delayed_completion (entry, 1);
-		} else if (entry->priv->completion)
-			e_entry_start_delayed_completion (entry, entry->priv->completion_delay);
+		} else if (entry->priv->completion_delay >= 0) {
+			int delay;
+			delay = entry->priv->popup_is_visible 
+				? 1 
+				: entry->priv->completion_delay;
+			e_entry_start_delayed_completion (entry, delay);
+		}
 	}
 	entry->priv->changed_since_keypress = FALSE;
 }
@@ -258,6 +275,8 @@ e_entry_init (GtkObject *object)
 	GtkTable *gtk_table = GTK_TABLE (object);
 
 	entry->priv = g_new0 (EEntryPrivate, 1);
+
+	entry->priv->emulate_label_resize = FALSE;
 	
 	entry->canvas = GNOME_CANVAS (e_canvas_new ());
 
@@ -277,6 +296,7 @@ e_entry_init (GtkObject *object)
 			   entry);
 
 	entry->priv->draw_borders = TRUE;
+	entry->priv->last_width = -1;
 
 	entry->item = E_TEXT(gnome_canvas_item_new(
 		gnome_canvas_root (entry->canvas),
@@ -286,8 +306,10 @@ e_entry_init (GtkObject *object)
 		"anchor", GTK_ANCHOR_NW,
 		"draw_borders", TRUE,
 		"draw_background", TRUE,
+		"draw_button", FALSE,
 		"max_lines", 1,
 		"editable", TRUE,
+		"allow_newlines", FALSE,
 		NULL));
 
 	gtk_signal_connect (GTK_OBJECT (entry->item),
@@ -455,36 +477,70 @@ e_entry_show_popup (EEntry *entry, gboolean visible)
 	if (pop == NULL)
 		return;
 
+	/* The async query can give us a result after the focus was lost by the
+	   widget.  In that case, we don't want to show the pop-up.   */
+	if (! GTK_WIDGET_HAS_FOCUS (entry->canvas))
+		return;
+
 	if (visible) {
 		GtkAllocation *dim = &(GTK_WIDGET (entry)->allocation);
-		gint x, y, fudge;
+		gint x, y, xo, yo, fudge;
 		const GdkEventMask grab_mask = (GdkEventMask)GDK_BUTTON_PRESS_MASK | GDK_BUTTON_MOTION_MASK | GDK_BUTTON_RELEASE_MASK;
 
 		/* Figure out where to put our popup. */
-		gdk_window_get_origin (GTK_WIDGET (entry)->window, &x, &y);
-		x += dim->x;
-		y += dim->height + dim->y;
+		gdk_window_get_origin (GTK_WIDGET (entry)->window, &xo, &yo);
+		x = xo + dim->x;
+		y = yo + dim->height + dim->y;
 
+#if MOVE_RIGHT_AND_UP
 		/* Put our popup slightly to the right and up, to try to give a visual cue that this popup
 		 is tied to this entry.  Otherwise one-row popups can sort of "blend" with an entry
 		 directly below. */
 		fudge = MAX (dim->height/10, 3); /* just in case we are using a really big font, etc. */
 		x += 2*fudge;
 		y -= fudge;
-
+#else
+		fudge = 1;
+		y -= fudge;
+#endif
 		gtk_widget_set_uposition (pop, x, y);
 		e_completion_view_set_width (E_COMPLETION_VIEW (entry->priv->completion_view), dim->width);
 
+#ifdef EVIL_POINTER_WARPING_HACK
+		/*
+		  I should have learned by now to listen to Havoc... 
+		  http://developer.gnome.org/doc/GGAD/faqs.html
+		*/
+		   
+		if (! entry->priv->popup_is_visible) {
+			GdkWindow *gwin = GTK_WIDGET (entry)->window;
+			gint xx, yy;
+			gdk_window_get_pointer (gwin, &xx, &yy, NULL);
+			xx += xo;
+			yy += yo;
+			
+			/* If we are inside the "zone of death" where the popup will appear, warp the pointer to safety.
+			   This is a horrible thing to do. */
+			if (y <= yy && yy < yy + dim->height && x <= xx && xx < xx + dim->width) {
+				XWarpPointer (GDK_WINDOW_XDISPLAY (gwin), None, GDK_WINDOW_XWINDOW (gwin),
+					      0, 0, 0, 0, 
+					      xx - xo, (y-1) - yo);
+			}
+		}
+#endif
+	
 		gtk_widget_show (pop);
 
 
-		if (! entry->priv->ptr_grab) {
-			entry->priv->ptr_grab = gdk_pointer_grab (GTK_WIDGET (entry->priv->completion_view)->window, TRUE,
-								  grab_mask, NULL, NULL, GDK_CURRENT_TIME);
-			if (entry->priv->ptr_grab)
-				gtk_grab_add (GTK_WIDGET (entry->priv->completion_view));
+		if (getenv ("GAL_E_ENTRY_NO_GRABS_HACK") == NULL && !entry->priv->ptr_grab) {
+			entry->priv->ptr_grab = (0 == gdk_pointer_grab (GTK_WIDGET (entry->priv->completion_view)->window, TRUE,
+									grab_mask, NULL, NULL, GDK_CURRENT_TIME));
+			if (entry->priv->ptr_grab) {
+ 				gtk_grab_add (GTK_WIDGET (entry->priv->completion_view));
+			}
 		}
-
+		
+		
 	} else {
 
 		gtk_widget_hide (pop);
@@ -497,12 +553,14 @@ e_entry_show_popup (EEntry *entry, gboolean visible)
 		entry->priv->ptr_grab = FALSE;
 
 		entry->priv->last_completion_pos = -1;
-
-
 	}
 
 	e_completion_view_set_editable (E_COMPLETION_VIEW (entry->priv->completion_view), visible);
-	entry->priv->popup_is_visible = visible;
+
+	if (entry->priv->popup_is_visible != visible) {
+		entry->priv->popup_is_visible = visible;
+		gtk_signal_emit (GTK_OBJECT (entry), e_entry_signals[E_ENTRY_COMPLETION_POPUP], (gint) visible);
+	}
 }
 
 static void
@@ -522,7 +580,7 @@ e_entry_start_completion (EEntry *entry)
 
 	if (e_entry_is_empty (entry))
 		return;
-	
+
 	e_completion_begin_search (entry->priv->completion,
 				   e_entry_get_text (entry),
 				   entry->priv->last_completion_pos = e_entry_get_position (entry),
@@ -579,16 +637,18 @@ static void
 full_cb (ECompletionView *view, gpointer user_data)
 {
 	EEntry *entry = E_ENTRY (user_data);
+	gboolean show;
 
-	e_entry_show_popup (entry, view->choice_count > 0);
+	show = GTK_WIDGET_HAS_FOCUS (GTK_WIDGET (entry->canvas)) && view->choices->len > 0;
+	e_entry_show_popup (entry, show);
 }
 
 static void
-browse_cb (ECompletionView *view, const gchar *txt, gpointer user_data)
+browse_cb (ECompletionView *view, ECompletionMatch *match, gpointer user_data)
 {
 	EEntry *entry = E_ENTRY (user_data);
-
-	if (txt == NULL) {
+	
+	if (match == NULL) {
 		/* Requesting a completion. */
 		e_entry_start_completion (entry);
 		return;
@@ -600,7 +660,7 @@ browse_cb (ECompletionView *view, const gchar *txt, gpointer user_data)
 	/* If there is no other handler in place, echo the selected completion in
 	   the entry. */
 	if (entry->priv->handler == NULL)
-		e_entry_set_text_quiet (entry, txt);
+		e_entry_set_text_quiet (entry, e_completion_match_get_match_text (match));
 }
 
 static void
@@ -621,7 +681,7 @@ unbrowse_cb (ECompletionView *view, gpointer user_data)
 }
 
 static void
-activate_cb (ECompletionView *view, const gchar *txt, gpointer extra_data, gpointer user_data)
+activate_cb (ECompletionView *view, ECompletionMatch *match, gpointer user_data)
 {
 	EEntry *entry = E_ENTRY (user_data);
 
@@ -632,9 +692,9 @@ activate_cb (ECompletionView *view, const gchar *txt, gpointer extra_data, gpoin
 	e_entry_show_popup (entry, FALSE);
 
 	if (entry->priv->handler)
-		entry->priv->handler (entry, txt, extra_data);
+		entry->priv->handler (entry, match);
 	else
-		e_entry_set_text (entry, txt);
+		e_entry_set_text (entry, match->match_text);
 
 	e_entry_cancel_delayed_completion (entry);
 }
@@ -668,6 +728,55 @@ button_press_cb (GtkWidget *w, GdkEvent *ev, gpointer user_data)
 	unbrowse_cb (E_COMPLETION_VIEW (w), entry);
 }
 
+static void
+cancel_completion_cb (ETextModel *model, gpointer user_data)
+{
+	EEntry *entry = E_ENTRY (user_data);
+
+	/* If we get the signal from the underlying text model, unbrowse.
+	   This usually means that the text model itself has done some
+	   sort of completion, or has otherwise transformed its contents
+	   in some way that would render any previous completion invalid. */
+	unbrowse_cb (E_COMPLETION_VIEW (entry->priv->completion_view), entry);
+}
+
+static gint
+key_press_cb (GtkWidget *w, GdkEventKey *ev, gpointer user_data)
+{
+	gint rv = 0;
+	/* Forward signal */
+	gtk_signal_emit_by_name (GTK_OBJECT (user_data), "key_press_event", ev, &rv);
+	return rv;
+}
+
+static gint
+key_release_cb (GtkWidget *w, GdkEventKey *ev, gpointer user_data)
+{
+	gint rv = 0;
+	/* Forward signal */
+	gtk_signal_emit_by_name (GTK_OBJECT (user_data), "key_release_event", ev, &rv);
+	return rv;
+}
+
+static void
+e_entry_make_completion_window_transient (EEntry *entry)
+{
+	GtkWidget *w;
+
+	if (entry->priv->have_set_transient || entry->priv->completion_view_popup == NULL)
+		return;
+	
+	w = GTK_WIDGET (entry)->parent;
+	while (w && ! GTK_IS_WINDOW (w))
+		w = w->parent;
+
+	if (w) {
+		gtk_window_set_transient_for (GTK_WINDOW (entry->priv->completion_view_popup),
+					      GTK_WINDOW (w));
+		entry->priv->have_set_transient = 1;
+	}
+}
+
 void
 e_entry_enable_completion_full (EEntry *entry, ECompletion *completion, gint delay, EEntryCompletionHandler handler)
 {
@@ -725,16 +834,39 @@ e_entry_enable_completion_full (EEntry *entry, ECompletion *completion, gint del
 							      entry);
 
 	entry->priv->completion_view_popup = gtk_window_new (GTK_WINDOW_POPUP);
-			    
+
+	e_entry_make_completion_window_transient (entry);
+
+	gtk_signal_connect (GTK_OBJECT (entry->item->model),
+			    "cancel_completion",
+			    GTK_SIGNAL_FUNC (cancel_completion_cb),
+			    entry);
+
+	gtk_signal_connect (GTK_OBJECT (entry->priv->completion_view_popup),
+			    "key_press_event",
+			    GTK_SIGNAL_FUNC (key_press_cb),
+			    entry->canvas);
+	gtk_signal_connect (GTK_OBJECT (entry->priv->completion_view_popup),
+			    "key_release_event",
+			    GTK_SIGNAL_FUNC (key_release_cb),
+			    entry->canvas);
+
+	e_completion_view_connect_keys (E_COMPLETION_VIEW (entry->priv->completion_view),
+					GTK_WIDGET (entry->canvas));
+
 	gtk_object_ref (GTK_OBJECT (entry->priv->completion_view_popup));
 	gtk_object_sink (GTK_OBJECT (entry->priv->completion_view_popup));
-	gtk_window_set_policy (GTK_WINDOW (entry->priv->completion_view_popup), FALSE, TRUE, FALSE);
+	gtk_window_set_policy (GTK_WINDOW (entry->priv->completion_view_popup), TRUE, TRUE, TRUE);
 	gtk_container_add (GTK_CONTAINER (entry->priv->completion_view_popup), entry->priv->completion_view);
 	gtk_widget_show (entry->priv->completion_view);
+}
 
-	e_completion_view_connect_keys (
-		E_COMPLETION_VIEW (entry->priv->completion_view),
-		GTK_WIDGET (entry->canvas));
+gboolean
+e_entry_completion_popup_is_visible (EEntry *entry)
+{
+	g_return_val_if_fail (E_IS_ENTRY (entry), FALSE);
+
+	return entry->priv->popup_is_visible;
 }
 
 /** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** ** **/
@@ -845,6 +977,16 @@ et_get_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 				NULL);
 		break;
 
+	case ARG_DRAW_BUTTON:
+		gtk_object_get (item,
+				"draw_button", &GTK_VALUE_BOOL (*arg),
+				NULL);
+		break;
+
+	case ARG_EMULATE_LABEL_RESIZE:
+		GTK_VALUE_BOOL (*arg) = entry->priv->emulate_label_resize;
+		break;
+
 	case ARG_CURSOR_POS:
 		gtk_object_get (item,
 				"cursor_pos", &GTK_VALUE_INT (*arg),
@@ -867,11 +1009,15 @@ et_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 	gint ythick;
 	GtkWidget *widget = GTK_WIDGET(entry->canvas);
 	
+	d(g_print("%s: arg_id: %d\n", __FUNCTION__, arg_id));
+
 	switch (arg_id){
 	case ARG_MODEL:
 		gtk_object_set(item,
 			       "model", GTK_VALUE_OBJECT (*arg),
 			       NULL);
+		if (entry->priv->emulate_label_resize)
+			gtk_widget_queue_resize (widget);
 		break;
 
 	case ARG_EVENT_PROCESSOR:
@@ -881,28 +1027,37 @@ et_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 		break;
 
 	case ARG_TEXT:
-
 		gtk_object_set(item,
 			       "text", GTK_VALUE_STRING (*arg),
 			       NULL);
+		d(g_print("%s: text: %s\n", __FUNCTION__, GTK_VALUE_STRING (*arg)));
+		if (entry->priv->emulate_label_resize)
+			gtk_widget_queue_resize (widget);
 		break;
 
 	case ARG_FONT:
 		gtk_object_set(item,
 			       "font", GTK_VALUE_STRING (*arg),
 			       NULL);
+		d(g_print("%s: font: %s\n", __FUNCTION__, GTK_VALUE_STRING (*arg)));
+		if (entry->priv->emulate_label_resize)
+			gtk_widget_queue_resize (widget);
 		break;
 
 	case ARG_FONTSET:
 		gtk_object_set(item,
 			       "fontset", GTK_VALUE_STRING (*arg),
 			       NULL);
+		if (entry->priv->emulate_label_resize)
+			gtk_widget_queue_resize (widget);
 		break;
 
 	case ARG_FONT_GDK:
 		gtk_object_set(item,
 			       "font_gdk", GTK_VALUE_BOXED (*arg),
 			       NULL);
+		if (entry->priv->emulate_label_resize)
+			gtk_widget_queue_resize (widget);
 		break;
 
 	case ARG_JUSTIFICATION:
@@ -974,30 +1129,40 @@ et_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 		gtk_object_set(item,
 			       "use_ellipsis", GTK_VALUE_BOOL (*arg),
 			       NULL);
+		if (entry->priv->emulate_label_resize)
+			gtk_widget_queue_resize (widget);
 		break;
 
 	case ARG_ELLIPSIS:
 		gtk_object_set(item,
 			       "ellipsis", GTK_VALUE_STRING (*arg),
 			       NULL);
+		if (entry->priv->emulate_label_resize)
+			gtk_widget_queue_resize (widget);
 		break;
 
 	case ARG_LINE_WRAP:
 		gtk_object_set(item,
 			       "line_wrap", GTK_VALUE_BOOL (*arg),
 			       NULL);
+		if (entry->priv->emulate_label_resize)
+			gtk_widget_queue_resize (widget);
 		break;
 		
 	case ARG_BREAK_CHARACTERS:
 		gtk_object_set(item,
 			       "break_characters", GTK_VALUE_STRING (*arg),
 			       NULL);
+		if (entry->priv->emulate_label_resize)
+			gtk_widget_queue_resize (widget);
 		break;
 
 	case ARG_MAX_LINES:
 		gtk_object_set(item,
 			       "max_lines", GTK_VALUE_INT (*arg),
 			       NULL);
+		if (entry->priv->emulate_label_resize)
+			gtk_widget_queue_resize (widget);
 		break;
 
 	case ARG_ALLOW_NEWLINES:
@@ -1006,16 +1171,15 @@ et_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 			       NULL);
 		break;
 
-	case ARG_DRAW_BORDERS: {
-		gboolean need_queue;
-		
-		need_queue = (entry->priv->draw_borders ^ GTK_VALUE_BOOL (*arg));
-		gtk_object_set (item, "draw_borders", GTK_VALUE_BOOL (*arg), NULL);
-		entry->priv->draw_borders = GTK_VALUE_BOOL (*arg);
-		if (need_queue)
+	case ARG_DRAW_BORDERS:
+		if (entry->priv->draw_borders != GTK_VALUE_BOOL (*arg)) {
+			entry->priv->draw_borders = GTK_VALUE_BOOL (*arg);
+			gtk_object_set (item,
+					"draw_borders", entry->priv->draw_borders,
+					NULL);
 			gtk_widget_queue_resize (GTK_WIDGET (entry));
+		}
 		break;
-	}
 
 	case ARG_CURSOR_POS:
 		gtk_object_set (item,
@@ -1026,6 +1190,18 @@ et_set_arg (GtkObject *o, GtkArg *arg, guint arg_id)
 		gtk_object_set (item, "draw_background",
 				GTK_VALUE_BOOL (*arg), NULL);
 		break;
+
+	case ARG_DRAW_BUTTON:
+		gtk_object_set (item, "draw_button",
+				GTK_VALUE_BOOL (*arg), NULL);
+		break;
+
+	case ARG_EMULATE_LABEL_RESIZE:
+		if (entry->priv->emulate_label_resize != GTK_VALUE_BOOL (*arg)) {
+			entry->priv->emulate_label_resize = GTK_VALUE_BOOL (*arg);
+			gtk_widget_queue_resize (widget);
+		}		
+		break;
 	}
 }
 
@@ -1039,8 +1215,10 @@ e_entry_destroy (GtkObject *object)
 
 	if (entry->priv->completion)
 		gtk_object_unref (GTK_OBJECT (entry->priv->completion));
-	if (entry->priv->completion_view_popup)
-		gtk_widget_destroy (entry->priv->completion_view_popup);
+	if (entry->priv->completion_view_popup) {
+		gtk_widget_destroy (GTK_WIDGET (entry->priv->completion_view_popup));
+		gtk_object_unref (GTK_OBJECT (entry->priv->completion_view_popup));
+	}
 	g_free (entry->priv->pre_browse_text);
 
 	if (entry->priv->changed_since_keypress_tag)
@@ -1051,12 +1229,34 @@ e_entry_destroy (GtkObject *object)
 
 	g_free (entry->priv);
 	entry->priv = NULL;
+
+	if (GTK_OBJECT_CLASS (parent_class)->destroy)
+		(* GTK_OBJECT_CLASS (parent_class)->destroy) (object);
+}
+
+static void
+e_entry_realize (GtkWidget *widget)
+{
+	EEntry *entry;
+
+	if (GTK_WIDGET_CLASS (parent_class)->realize)
+		(* GTK_WIDGET_CLASS (parent_class)->realize) (widget);
+
+	entry = E_ENTRY (widget);
+
+	e_entry_make_completion_window_transient (entry);
+
+	if (entry->priv->emulate_label_resize) {
+		d(g_print("%s: queue_resize\n", __FUNCTION__));
+		gtk_widget_queue_resize (GTK_WIDGET (entry->canvas));
+	}
 }
 
 static void
 e_entry_class_init (GtkObjectClass *object_class)
 {
 	EEntryClass *klass = E_ENTRY_CLASS(object_class);
+	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS(object_class);
 
 	parent_class = gtk_type_class (PARENT_TYPE);
 
@@ -1064,32 +1264,45 @@ e_entry_class_init (GtkObjectClass *object_class)
 	object_class->get_arg = et_get_arg;
 	object_class->destroy = e_entry_destroy;
 
+	widget_class->realize = e_entry_realize;
+
 	klass->changed = NULL;
 	klass->activate = NULL;
 
-	e_entry_signals[E_ENTRY_CHANGED] = gtk_signal_new ("changed",
-							   GTK_RUN_LAST,
-							   object_class->type,
-							   GTK_SIGNAL_OFFSET (EEntryClass, changed),
-							   gtk_marshal_NONE__NONE,
-							   GTK_TYPE_NONE, 0);
+	e_entry_signals[E_ENTRY_CHANGED] =
+		gtk_signal_new ("changed",
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (object_class),
+				GTK_SIGNAL_OFFSET (EEntryClass, changed),
+				gtk_marshal_NONE__NONE,
+				GTK_TYPE_NONE, 0);
 
-	e_entry_signals[E_ENTRY_ACTIVATE] = gtk_signal_new ("activate",
-							    GTK_RUN_LAST,
-							    object_class->type,
-							    GTK_SIGNAL_OFFSET (EEntryClass, activate),
-							    gtk_marshal_NONE__NONE,
-							    GTK_TYPE_NONE, 0);
+	e_entry_signals[E_ENTRY_ACTIVATE] =
+		gtk_signal_new ("activate",
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (object_class),
+				GTK_SIGNAL_OFFSET (EEntryClass, activate),
+				gtk_marshal_NONE__NONE,
+				GTK_TYPE_NONE, 0);
 
-	e_entry_signals[E_ENTRY_POPUP] = gtk_signal_new ("popup",
-							 GTK_RUN_LAST,
-							 object_class->type,
-							 GTK_SIGNAL_OFFSET (EEntryClass, popup),
-							 gtk_marshal_NONE__POINTER_INT,
-							 GTK_TYPE_NONE, 2, GTK_TYPE_POINTER, GTK_TYPE_INT);
+	e_entry_signals[E_ENTRY_POPUP] =
+		gtk_signal_new ("popup",
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (object_class),
+				GTK_SIGNAL_OFFSET (EEntryClass, popup),
+				gtk_marshal_NONE__POINTER_INT,
+				GTK_TYPE_NONE, 2, GTK_TYPE_POINTER, GTK_TYPE_INT);
+
+	e_entry_signals[E_ENTRY_COMPLETION_POPUP] =
+		gtk_signal_new ("completion_popup",
+				GTK_RUN_LAST,
+				E_OBJECT_CLASS_TYPE (object_class),
+				GTK_SIGNAL_OFFSET (EEntryClass, completion_popup),
+				gtk_marshal_NONE__INT,
+				GTK_TYPE_NONE, 1, GTK_TYPE_INT);
 
 
-	gtk_object_class_add_signals (object_class, e_entry_signals, E_ENTRY_LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, e_entry_signals, E_ENTRY_LAST_SIGNAL);
 
 	gtk_object_add_arg_type ("EEntry::model",
 				 GTK_TYPE_OBJECT, GTK_ARG_READWRITE, ARG_MODEL);  
@@ -1131,6 +1344,10 @@ e_entry_class_init (GtkObjectClass *object_class)
 				 GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_DRAW_BORDERS);
 	gtk_object_add_arg_type ("EEntry::draw_background",
 				 GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_DRAW_BACKGROUND);
+	gtk_object_add_arg_type ("EEntry::draw_button",
+				 GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_DRAW_BUTTON);
+	gtk_object_add_arg_type ("EEntry::emulate_label_resize",
+				 GTK_TYPE_BOOL, GTK_ARG_READWRITE, ARG_EMULATE_LABEL_RESIZE);
 	gtk_object_add_arg_type ("EEntry::cursor_pos",
 				 GTK_TYPE_INT, GTK_ARG_READWRITE, ARG_CURSOR_POS);
 }
