@@ -229,12 +229,15 @@ gint      g_unichar_to_utf8 (gunichar    c,
  */
 
 gboolean g_utf8_validate (const gchar  *str,
-                          gint          max_len,
+                          gssize      max_len,
                           const gchar **end);
 
 /* Validate a Unicode character */
 gboolean g_unichar_validate (gunichar ch);
 
+gunichar g_utf8_get_char_validated (const   gchar *p,
+				    gssize  max_len);
+
 gchar *g_ucs4_to_utf8 (const gunichar *str,
                        glong           len,              
                        glong          *items_read,       
