@@ -1,32 +1,31 @@
-/* GTK - The GIMP Toolkit
- * Copyright (C) 1995-1997 Peter Mattis, Spencer Kimball and Josh MacDonald
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* 
+ * e-scroll-frame.c
+ * Copyright 2000, 2001, Ximian, Inc.
  *
- * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * Authors:
+ *   Federico Mena <federico@ximian.com>
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * EScrollFrame based on GtkScrolledWindow.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the
- * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
- */
-
-/*
  * Modified by the GTK+ Team and others 1997-1999.  See the AUTHORS
  * file for a list of people on the GTK+ Team.  See the ChangeLog
  * files for a list of changes.  These files are distributed with
  * GTK+ at ftp://ftp.gtk.org/pub/gtk/.
- */
-
-/* EScrollFrame based on GtkScrolledWindow.
  *
- * Modifications by Federico Mena <federico@helixcode.com>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Library General Public
+ * License, version 2, as published by the Free Software Foundation.
+ *
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Library General Public License for more details.
+ *
+ * You should have received a copy of the GNU Library General Public
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
 #include <config.h>
@@ -925,7 +924,8 @@ adjustment_changed (GtkAdjustment *adj, gpointer data)
 	sf = E_SCROLL_FRAME (data);
 	priv = sf->priv;
 
-	if (adj == gtk_range_get_adjustment (GTK_RANGE (priv->hsb))) {
+	if (priv->hsb &&
+	    adj == gtk_range_get_adjustment (GTK_RANGE (priv->hsb))) {
 		if (priv->hsb_policy == GTK_POLICY_AUTOMATIC) {
 			gboolean visible;
 
@@ -934,7 +934,8 @@ adjustment_changed (GtkAdjustment *adj, gpointer data)
 			if (priv->hsb_visible != visible)
 				gtk_widget_queue_resize (GTK_WIDGET (sf));
 		}
-	} else if (adj == gtk_range_get_adjustment (GTK_RANGE (priv->vsb))) {
+	} else if (priv->vsb &&
+		   adj == gtk_range_get_adjustment (GTK_RANGE (priv->vsb))) {
 		if (priv->vsb_policy == GTK_POLICY_AUTOMATIC) {
 			gboolean visible;
 
