@@ -1,28 +1,31 @@
-/* mode: c; c-basic-offset: 8 -*- */
-/*
- * gtk-combo-stack: A combo box for displaying stacks (useful for Undo lists)
+/* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
+/* 
+ * gtk-combo-stack.c - A combo box for displaying stacks (useful for Undo lists)
  *
  * Copyright (C) 2000 RDI Gerg <cactus@cactus.rulez.org>
  *
+ * Authors:
+ *   RDI Gerg <cactus@cactus.rulez.org>
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
- * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * License, version 2, as published by the Free Software Foundation.
  *
- * This library is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * This library is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Library General Public License for more details.
  *
  * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * License along with this library; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
+ * 02111-1307, USA.
  */
 
-#include <gnome.h>
-#include <gtk/gtk.h>
+#include <libgnomeui/gnome-preferences.h>
 #include <libgnomeui/gnome-stock.h>
 #include "gtk-combo-stack.h"
+#include "gal/util/e-util.h"
 
 static GtkObjectClass *gtk_combo_stack_parent_class;
 
@@ -61,12 +64,12 @@ gtk_combo_stack_class_init (GtkObjectClass *object_class)
 	gtk_combo_stack_signals [POP] = gtk_signal_new (
 		"pop",
 		GTK_RUN_LAST,
-		object_class->type,
+		E_OBJECT_CLASS_TYPE (object_class),
 		GTK_SIGNAL_OFFSET (GtkComboBoxClass, pop_down_done),
 		gtk_marshal_NONE__INT,
 		GTK_TYPE_NONE, 1, GTK_TYPE_INT);
 	
-	gtk_object_class_add_signals (object_class, gtk_combo_stack_signals, LAST_SIGNAL);
+	E_OBJECT_CLASS_ADD_SIGNALS (object_class, gtk_combo_stack_signals, LAST_SIGNAL);
 }
 
 static void
